/***************************************************************************

  machine.c

  Functions to emulate general aspects of the machine (RAM, ROM, interrupts,
  I/O ports)

***************************************************************************/

#include "driver.h"
#include "M6502.h"


#define IN1_VBLANK (1<<7)


static int vblank;


int btime_init_machine(const char *gamename)
{
#if 0
        /* Encryption is a bit shuffle on some opcodes like this:

           7 6 5 4 3 2 1 0
           | | | | | | | |
           2 7 6 4 5 3 1 0

           Not all opcodes are encrypted.  It seems only opcodes at
           location xxxxxxx1xxxxx1xx are.  But even if the address is
           in the form above, it doesn't mean the opcode is encrypted.
           So maybe there is some other info.  Information provided by
           Kevin Brisley
        */


	/* patch the encrypted roms */

        RAM[0xb105] = 0xa5;
        RAM[0xb116] = 0xea;
        RAM[0xb11e] = 0xea;
        RAM[0xb146] = 0xea;
        RAM[0xb14d] = 0xea;
        RAM[0xb154] = 0xea;
        RAM[0xb175] = 0xea;
        RAM[0xb18f] = 0xea;
        RAM[0xb195] = 0x60;
        RAM[0xb19d] = 0xea;
        RAM[0xb1a4] = 0xe8;
        RAM[0xb1bd] = 0xea;
        RAM[0xb1c4] = 0xa9;
        RAM[0xb1cf] = 0xc8;
        RAM[0xb1d4] = 0xa5;
        RAM[0xb1f4] = 0xe8;
        RAM[0xb336] = 0xea;
        RAM[0xb396] = 0x85;
        RAM[0xb39e] = 0xc8;
        RAM[0xb3ad] = 0x85;
        RAM[0xb3af] = 0xea;
        RAM[0xb3bc] = 0xa5;
        RAM[0xb3cc] = 0xa9;
        RAM[0xb507] = 0xa9;
        RAM[0xb50c] = 0xa9;
        RAM[0xb516] = 0x60;
        RAM[0xb525] = 0x8d;
        RAM[0xb52d] = 0xa9;
        RAM[0xb537] = 0xa9;
        RAM[0xb53c] = 0xa9;
        RAM[0xb575] = 0x8d;
        RAM[0xb57d] = 0xa9;
        RAM[0xb587] = 0xa9;
        RAM[0xb58c] = 0xa9;
        RAM[0xb59e] = 0xa9;
        RAM[0xb5ac] = 0xea;
        RAM[0xb5b6] = 0x18;
        RAM[0xb5be] = 0x20;
        RAM[0xb5d4] = 0xd8;
        RAM[0xb5df] = 0xa5;
        RAM[0xb5e5] = 0xea;
        RAM[0xb5e6] = 0xa5;
        RAM[0xb5fd] = 0x85;
        RAM[0xb5ff] = 0xea;
        RAM[0xb707] = 0xe8;
        RAM[0xb70d] = 0xa5;
        RAM[0xb715] = 0xea;
        RAM[0xb71f] = 0xa5;
        RAM[0xb7fe] = 0xa2;
        RAM[0xb90e] = 0xea;
        RAM[0xb91e] = 0xea;
        RAM[0xb92d] = 0x85;
        RAM[0xb92f] = 0x85;
        RAM[0xb937] = 0xa9;
        RAM[0xb93f] = 0xa9;
        RAM[0xb947] = 0xa9;
        RAM[0xb965] = 0xa9;
        RAM[0xb96f] = 0xa9;
        RAM[0xb974] = 0xa9;
        RAM[0xb97e] = 0xa9;
        RAM[0xb986] = 0xa2;
        RAM[0xb98c] = 0xea;
        RAM[0xb99f] = 0xa5;
        RAM[0xb9a5] = 0xea;
        RAM[0xb9bc] = 0xea;
        RAM[0xb9ce] = 0xea;
        RAM[0xb9f7] = 0xea;
        RAM[0xc10e] = 0xa9;
        RAM[0xc115] = 0xea;
        RAM[0xc15e] = 0xa5;
        RAM[0xc17f] = 0x20;
        RAM[0xc347] = 0xc8;
        RAM[0xc34d] = 0xea;
        RAM[0xc355] = 0x8d;
        RAM[0xc35c] = 0x8d;
        RAM[0xc35f] = 0x60;
        RAM[0xc365] = 0xea;
        RAM[0xc36e] = 0xa2;
        RAM[0xc377] = 0xea;
        RAM[0xc37c] = 0xc8;
        RAM[0xc386] = 0xea;
        RAM[0xc396] = 0x9d;
        RAM[0xc3a4] = 0xa9;
        RAM[0xc3ac] = 0xea;
        RAM[0xc3b7] = 0xea;
        RAM[0xc3bc] = 0xea;
        RAM[0xc3d6] = 0x85;
        RAM[0xc504] = 0xe8;
        RAM[0xc50c] = 0xa2;
        RAM[0xc51e] = 0xea;
        RAM[0xc525] = 0x8d;
        RAM[0xc52e] = 0x8d;
        RAM[0xc534] = 0x8d;
        RAM[0xc537] = 0x8d;
        RAM[0xc53d] = 0x8d;
        RAM[0xc546] = 0x8d;
        RAM[0xc54c] = 0x8d;
        RAM[0xc54f] = 0x60;
        RAM[0xc557] = 0x85;
        RAM[0xc55e] = 0x20;
        RAM[0xc56e] = 0xa9;
        RAM[0xc57d] = 0xa9;
        RAM[0xc587] = 0xa9;
        RAM[0xc58c] = 0xa9;
        RAM[0xc594] = 0x85;
        RAM[0xc596] = 0xa9;
        RAM[0xc59c] = 0xa9;
        RAM[0xc5a6] = 0x20;
        RAM[0xc7dd] = 0xea;
        RAM[0xc7ee] = 0xea;
        RAM[0xc7fd] = 0x85;
        RAM[0xc7ff] = 0x85;
        RAM[0xc92f] = 0xa9;
        RAM[0xc937] = 0xa9;
        RAM[0xc93c] = 0xa9;
        RAM[0xc946] = 0xa9;
        RAM[0xc955] = 0xa9;
        RAM[0xc95d] = 0xa9;
        RAM[0xc966] = 0xa9;
        RAM[0xc96f] = 0x85;
        RAM[0xc976] = 0xea;
        RAM[0xc97f] = 0x85;
        RAM[0xc986] = 0xc8;
        RAM[0xcb17] = 0x20;
        RAM[0xcb1e] = 0xea;
        RAM[0xcb34] = 0xa9;
        RAM[0xcb44] = 0xea;
        RAM[0xcb56] = 0xea;
        RAM[0xcb5e] = 0xea;
        RAM[0xcb6e] = 0xea;
        RAM[0xcb76] = 0xc8;
        RAM[0xcb7e] = 0xa6;
        RAM[0xcb9e] = 0xea;
        RAM[0xcbad] = 0xa9;
        RAM[0xcbb5] = 0xea;
        RAM[0xcbe4] = 0xea;
        RAM[0xcbe7] = 0x98;
        RAM[0xcd0e] = 0xa0;
        RAM[0xcd14] = 0xea;
        RAM[0xcd1e] = 0xc8;
        RAM[0xcd2d] = 0x4c;
        RAM[0xcd3e] = 0xea;
        RAM[0xcd47] = 0xa9;
        RAM[0xcd4d] = 0xea;
        RAM[0xcd56] = 0xc8;
        RAM[0xcd7c] = 0xea;
        RAM[0xcd8e] = 0xa5;
        RAM[0xcda4] = 0xbd;
        RAM[0xcdaf] = 0x46;
        RAM[0xcdb5] = 0xa9;
        RAM[0xcdbf] = 0x06;
        RAM[0xcdc5] = 0x26;
        RAM[0xcdc7] = 0x18;
        RAM[0xcdce] = 0xa5;
        RAM[0xcdd4] = 0xa5;
        RAM[0xd11d] = 0xa5;
        RAM[0xd124] = 0xa9;
        RAM[0xd137] = 0xea;
        RAM[0xd13c] = 0xea;
        RAM[0xd15d] = 0xea;
        RAM[0xd19f] = 0x85;
        RAM[0xd1a7] = 0xa9;
        RAM[0xd306] = 0x98;
        RAM[0xd30d] = 0xea;
        RAM[0xd334] = 0xea;
        RAM[0xd36c] = 0xea;
        RAM[0xd394] = 0x4c;
        RAM[0xd3ac] = 0x85;
        RAM[0xd3ae] = 0xea;
        RAM[0xd3b4] = 0xb5;
        RAM[0xd3bf] = 0xb9;
        RAM[0xd3c4] = 0xa4;
        RAM[0xd3de] = 0xb5;
        RAM[0xd3e6] = 0xb5;
        RAM[0xd3ec] = 0x86;
        RAM[0xd3ee] = 0xa4;
        RAM[0xd3ff] = 0x18;
        RAM[0xd526] = 0xa9;
        RAM[0xd52c] = 0xea;
        RAM[0xd557] = 0xa9;
        RAM[0xd55d] = 0xea;
        RAM[0xd57e] = 0xa5;
        RAM[0xd597] = 0x68;
        RAM[0xd5ce] = 0xa5;
        RAM[0xd5e7] = 0x68;
        RAM[0xd72d] = 0xa9;
        RAM[0xd734] = 0xa5;
        RAM[0xd744] = 0xea;
        RAM[0xd757] = 0xa5;
        RAM[0xd764] = 0xea;
        RAM[0xd90f] = 0x85;
        RAM[0xd92e] = 0x85;
        RAM[0xd937] = 0x60;
        RAM[0xd94d] = 0x85;
        RAM[0xd94f] = 0xea;
        RAM[0xd956] = 0x60;
        RAM[0xd95e] = 0xa9;
        RAM[0xd965] = 0xea;
        RAM[0xd9b7] = 0xad;
        RAM[0xd9c5] = 0x18;
        RAM[0xd9cc] = 0xa5;
        RAM[0xd9d7] = 0xa0;
        RAM[0xd9f7] = 0xea;
        RAM[0xdb1d] = 0xbd;
        RAM[0xdb45] = 0x96;
        RAM[0xdb4d] = 0xa9;
        RAM[0xdb5c] = 0x46;
        RAM[0xdb6d] = 0xea;
        RAM[0xdb77] = 0x18;
        RAM[0xdb8c] = 0xea;
        RAM[0xdba5] = 0xbd;
        RAM[0xdbaf] = 0xa9;
        RAM[0xdbb5] = 0x85;
        RAM[0xdbb5] = 0x85;
        RAM[0xdbb7] = 0x85;
        RAM[0xdbbd] = 0xa6;
        RAM[0xdbd7] = 0x88;
        RAM[0xdbe5] = 0x88;
        RAM[0xdbed] = 0xad;
        RAM[0xdd1c] = 0xea;
        RAM[0xdd2e] = 0xea;
        RAM[0xdd6d] = 0xea;
        RAM[0xdd8d] = 0xa2;
        RAM[0xddae] = 0x85;
        RAM[0xddb6] = 0xea;
        RAM[0xddbd] = 0xea;
        RAM[0xdddc] = 0x84;
        RAM[0xddde] = 0x20;
        RAM[0xddee] = 0xea;
        RAM[0xddfd] = 0xb9;
        RAM[0xdf0f] = 0xea;
        RAM[0xdf1c] = 0x4c;
        RAM[0xdf67] = 0xa5;
        RAM[0xdfd6] = 0x29;
        RAM[0xe12e] = 0xa9;
        RAM[0xe134] = 0xea;
        RAM[0xe15c] = 0xea;
        RAM[0xe167] = 0x20;
        RAM[0xe16d] = 0xea;
        RAM[0xe17e] = 0xea;
        RAM[0xe18f] = 0xea;
        RAM[0xe1b5] = 0xea;
        RAM[0xe1be] = 0xe6;
        RAM[0xe1fe] = 0xea;
        RAM[0xe305] = 0xea;
        RAM[0xe315] = 0x4c;
        RAM[0xe32d] = 0x20;
        RAM[0xe347] = 0x99;
        RAM[0xe355] = 0x85;
        RAM[0xe357] = 0xea;
        RAM[0xe36c] = 0xb9;
        RAM[0xe38d] = 0x85;
        RAM[0xe38f] = 0xea;
        RAM[0xe3a4] = 0xb9;
        RAM[0xe3c5] = 0x85;
        RAM[0xe3c7] = 0xea;
        RAM[0xe3dc] = 0xb9;
        RAM[0xe3fd] = 0x85;
        RAM[0xe3ff] = 0xea;
        RAM[0xe51c] = 0xb9;
        RAM[0xe524] = 0xb9;
        RAM[0xe53e] = 0xb9;
        RAM[0xe546] = 0xa9;
        RAM[0xe567] = 0xf0;
        RAM[0xe5ac] = 0xea;
        RAM[0xe5b5] = 0xea;
        RAM[0xe5be] = 0xea;
        RAM[0xe5c7] = 0xea;
        RAM[0xe5cc] = 0x85;
        RAM[0xe5ce] = 0xea;
        RAM[0xe704] = 0xea;
        RAM[0xe70f] = 0xea;
        RAM[0xe72e] = 0xa9;
        RAM[0xe737] = 0xea;
        RAM[0xe746] = 0x86;
        RAM[0xe75e] = 0xa4;
        RAM[0xe764] = 0x4c;
        RAM[0xe775] = 0xea;
        RAM[0xe784] = 0xea;
        RAM[0xe7bf] = 0xa4;
        RAM[0xe7e7] = 0xea;
        RAM[0xe90f] = 0xa9;
        RAM[0xe916] = 0xea;
        RAM[0xe947] = 0xa5;
        RAM[0xe95e] = 0xea;
        RAM[0xe96f] = 0xea;
        RAM[0xe986] = 0xea;
        RAM[0xe99d] = 0x8a;
        RAM[0xe9a4] = 0xa9;
        RAM[0xe9f5] = 0xea;
        RAM[0xeb75] = 0x85;
        RAM[0xeb77] = 0xea;
        RAM[0xebad] = 0xea;
        RAM[0xebbd] = 0xea;
        RAM[0xebff] = 0x4c;
        RAM[0xf1af] = 0xea;
        RAM[0xf1b4] = 0x85;
        RAM[0xf1b6] = 0xea;
        RAM[0xf1bf] = 0xa9;
        RAM[0xf1c5] = 0xea;
        RAM[0xf1e4] = 0xa5;
        RAM[0xf1ec] = 0xf8;
        RAM[0xff1e] = 0xad;
#endif
	return 0;
}



int btime_DSW1_r(int offset)
{
	int res;


	res = readinputport(3);

	if (vblank == 0) res |= IN1_VBLANK;

	return res;
}



/***************************************************************************

  Burger Time doesn't have VBlank interrupts; the software polls port IN0 to
  know when a vblank is happening. Therefore we set a flag here, to let
  btime_DSW1_r() know when it has to report a vblank.

***************************************************************************/
int btime_interrupt(void)
{
	/* let btime_DSW1_r() know when it is time to report a vblank */
	/* I'm not yet sure about how the vertical blanking should be handled. */
	/* I think that IN1_VBLANK should be 1 during the whole vblank, which */
	/* should last roughly 1/12th of the frame. */
	vblank = (vblank + 1) % 12;

	/* IRQs are used to check coin insertion and diagnostic commands */
	return INT_IRQ;
}
